/*
  This example was written in CVI version 5.5.1 in Win2KSP1
  
  It used a KPCI-PDISO8A with kpciiso-850A01 (initial release) of the driver
  
  Important Compiler Defines for the project:  /DWIN32_LEAN_AND_MEAN  /D_MSC_VER=0
  
  This example is suitable also for KPCI-PIO32IOA with minor modification.
  
  Channels of KPCI-PDISO8A:
  Digital input:
               0 : the 8 isolated inputs
               1:  a virtual channel to read back the state of the relay outputs
 Digital output:
               1 : the 8 relays
               
  Channels of KPCI-PIO32IOA:
  Digital input:
               0 : the 8 isolated inputs on first connector
               1 : the 8 isolated inputs on second connector
               2 : a virtual channel to read back the state of the relay outputs on first connector
               3 : a virtual channel to read back the state of the relay outputs on second connector
               
 Digital output:
               2 : the 8 relays on first connector  (upper)
               3 : the 8 relays on second connector (lower)
               
*/

static int stat;
#include <cvirte.h>		/* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "sample.h"
#include <windows.h>
#include <ansi_c.h>
#include "drvlinx.h"
#include "dlcodes.h"
static int panelHandle;


HWND window;

HINSTANCE driverInstance; // need a handle for our instance of the open driver

DL_SERVICEREQUEST *pSR;  // declare a service request structure     

	
int main (int argc, char *argv[])
{
	
	if (InitCVIRTE (0, argv, 0) == 0)	/* Needed if linking in external compiler; harmless otherwise */
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "sample.uir", PANEL)) < 0)
		return -1;
	DisplayPanel (panelHandle);
	RunUserInterface ();
	return 0;
}


int CVICALLBACK quit (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
		    if (pSR != NULL)
		    {
			free(pSR);
			pSR = NULL;
			}
			CloseDriverLINX(driverInstance);
			QuitUserInterface (0);
			
			break;
	}
	return 0;
}

int CVICALLBACK Initialize (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			window=FindWindow(NULL,"KPCI-ISO Driver Example"); //Get the Window handle from the CVI panel
			driverInstance=OpenDriverLINX(window,"kpciiso");  //Pass the window handle to open the driver
			pSR=(DL_SERVICEREQUEST*)malloc(sizeof(DL_SERVICEREQUEST));  //Allocate space for the Service Request
			memset(pSR,0,sizeof(DL_SERVICEREQUEST));   //Initialize the Service Request members
			DL_SetServiceRequestSize(*pSR);    	//Setup the Service Request for usage
			pSR->hWnd=window;                	//Set the Widows handle property
			pSR->device=0; 						//Set the device property, in this case it is device 0	
			pSR->subsystem=DEVICE;				//We are commnicating with the DEVICE, not AI, AO, etc..																		
			pSR->mode=OTHER;					//The mode is OTHER, not DMA or INTerrupt
			pSR->operation=INITIALIZE;			//The operation is to INITIALIZE the driver													 
			DriverLINX(pSR);					//Execute the Service Request
			
			if(pSR->result!=NoErr)				//Check for errors
			{
				pSR->operation=MESSAGEBOX;
				DriverLINX(pSR);
			}
			break;
	}
	return 0;
}

int CVICALLBACK read_di (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			pSR->subsystem = DI;				//Talk to the Digital Input subsystem
			pSR->mode = POLLED;					//The mode is POLLED, not DMA ot INTerrupt
			pSR->channels.nChannels=1;			//The total number of channels is 1
			pSR->channels.chanGain[0].channel=0;//We are talking to channel 0
			pSR->status.typeStatus=IOVALUE;		//We are getting an I/O value
			pSR->operation = START;		    	//The operation is to START
			DriverLINX(pSR);					//Execute the Service Request

			if(pSR->result!=NoErr)				//Check for errors
			{
				pSR->operation=MESSAGEBOX;
				DriverLINX(pSR);
			}
			else
			{
			 // display the value
			 SetCtrlVal (panel, PANEL_NUMERIC, pSR->status.u.ioValue);
			 // a polled mode single sample can live in status.u.ioValue member 
			 }

		break;
		}
	return 0;
}

int CVICALLBACK digout (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event) {
		case EVENT_COMMIT:
			pSR->mode=POLLED;					//The mode is POLLED I/O
			pSR->subsystem = DO;
			pSR->channels.nChannels=1;			//The total number of channels is 1
			pSR->channels.chanGain[0].channel=1;//We are talking to channel 1
			pSR->status.typeStatus=IOVALUE;		//We are sending an I/O value
			pSR->status.u.ioValue=255;			//We are sending a value of 255,i.e. all 8 lines are high
			pSR->operation = START;				//The operationi is to START
			
			DriverLINX(pSR);					//Execute the Service Request
			
			if(pSR->result!=NoErr)				//Check for errors
			{
				pSR->operation=MESSAGEBOX;
				DriverLINX(pSR);
			}
			break;
	}
	return 0;
}

int CVICALLBACK allLow (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
		    pSR->mode=POLLED;					//The mode is POLLED I/O
		    pSR->subsystem = DO;
			pSR->channels.nChannels=1;			//The total number of channels is 1
			pSR->channels.chanGain[0].channel=1;//We are talking to channel 1
			pSR->status.typeStatus=IOVALUE;		//We are sending an I/O value
			pSR->status.u.ioValue=0;			//We are sending a value of 0
			pSR->operation = START;				//The operationi is to START
			
			DriverLINX(pSR);					//Execute the Service Request
			
			if(pSR->result!=NoErr)				//Check for errors
			{
				pSR->operation=MESSAGEBOX;
				DriverLINX(pSR);
			}
			break;
		}
	return 0;
}
